#version 330

in vec4 ex_Color;
in vec2 ex_TextCord;
in vec3 ex_FragWorldPos;
in vec3 ex_camVector;
in vec3 ex_lightVector;

out vec4 out_Color;

uniform vec3 camWorldPos;
uniform vec3 ambientColor;
uniform float ambientPower;
uniform vec4 lightColor;

uniform float shininess;
uniform float cutPlaneHeight;
uniform int cutPlaneMode; // 0 disabled; 1 positive discard; -1 negativ discard;

uniform sampler2D texDiffuse;
uniform sampler2D nomalMap;

void main(void)
{
	if(cutPlaneMode!=0)
	{
		if(cutPlaneMode>0 && ex_FragWorldPos.y<cutPlaneHeight) discard;
		if(cutPlaneMode<0 && ex_FragWorldPos.y>cutPlaneHeight) discard;
	}
	vec4 textureColor = texture2D(texDiffuse,ex_TextCord);
	
	
	if(lightColor.a==0)
	{
		out_Color = (vec4(ambientColor*ambientPower,1))*textureColor;
		out_Color.a = min(1,(cutPlaneHeight-ex_FragWorldPos.y)/normalize(camWorldPos-ex_FragWorldPos).y/100);
		return;
	}

	
	vec3 n = normalize(texture2D(nomalMap,vec2(ex_TextCord.x,ex_TextCord.y)).rgb*2.0-1.0);
	vec3 l = normalize(ex_lightVector);
	vec3 c = normalize(ex_camVector);	
	vec3 r = reflect(-l,n);
	float cosAlpha = 0;
	if(shininess!=0)
	{
		 cosAlpha=pow(clamp(dot(c,r),0,1),shininess*5)* lightColor.a;
	}
	float cosTheta = clamp(dot(n,l),0,1)* lightColor.a;

	
	out_Color = (vec4(lightColor.xyz*cosTheta*0.7,1)+vec4(lightColor.xyz*cosAlpha,1))*textureColor;
	            
	out_Color.a=0;
}