#version 330

in vec4 in_Position;
in vec4 in_Color;
in vec2 in_TextCord;
in vec3 in_Tangent;
in vec3 in_Bitangent;
in vec3 in_Normal;

out vec4 ex_Color;
out vec2 ex_TextCord;
out vec3 ex_FragWorldPos;
out vec3 ex_camVector;
out vec3 ex_lightVector;

uniform mat4 ModelMatrix;
uniform mat4 ViewMatrix;
uniform mat4 ProjectionMatrix;
uniform vec3 lightPos;

void main(void)
{
	gl_Position = (ProjectionMatrix*ViewMatrix*ModelMatrix)*in_Position;
	ex_Color = in_Color;
	ex_TextCord = in_TextCord;
	
	ex_FragWorldPos = (ModelMatrix*in_Position).xyz;
	vec3 CamFragPos = ((ViewMatrix*ModelMatrix)*in_Position).xyz;
	vec3 CamLightPos = ((ViewMatrix)*vec4(lightPos,1)).xyz;
	
	// Camera Space
	vec3 camVector = vec3(0,0,0)-CamFragPos;
	vec3 lightVector = CamLightPos+camVector;	
	vec3 vertexTangent = normalize(mat3(ViewMatrix*ModelMatrix)* in_Tangent);
	vec3 vertexBitangent = normalize(mat3(ViewMatrix*ModelMatrix)* in_Bitangent);
	vec3 vertexNormal = normalize(mat3(ViewMatrix*ModelMatrix)* in_Normal);
	
	// Tangent Space
	mat3 TBN = transpose(mat3(vertexTangent,vertexBitangent,vertexNormal));
	ex_camVector = TBN*camVector;
	ex_lightVector = TBN*lightVector;
}
