#version 330

layout (triangles_adjacency) in;
layout (triangle_strip, max_vertices = 18) out;

in vec4 ex_Pos[6];
in vec4 ex_Normal[6];

uniform int robust;
uniform vec3 lightPos;
uniform mat4 ViewMatrix;
uniform mat4 ProjectionMatrix;
uniform int isMirrored;

float epsilon = 0.1;
void main(void)
{
	mat4 VPm= ProjectionMatrix*ViewMatrix;
	
	vec3 correctPos[6];
	correctPos[0]=ex_Pos[0].xyz;
	correctPos[1]=ex_Pos[1].xyz;
	correctPos[2]=ex_Pos[2].xyz;
	correctPos[3]=ex_Pos[3].xyz;
	correctPos[4]=ex_Pos[4].xyz;
	correctPos[5]=ex_Pos[5].xyz;
	

	vec3 correctNrom0=-ex_Normal[0].xyz;
	vec3 correctNrom2=-ex_Normal[2].xyz;
	vec3 correctNrom4=-ex_Normal[4].xyz;

	
	vec3 edge02=normalize(ex_Pos[2].xyz-ex_Pos[0].xyz);
	vec3 edge04=normalize(ex_Pos[4].xyz-ex_Pos[0].xyz);
	
	vec3 normalPrim= cross(edge02,edge04);
	vec3 lightDir=lightPos-ex_Pos[0].xyz;
	float dporduct=dot(lightDir,normalPrim);
	if(robust>0 && ((isMirrored==0 && dporduct<=0) || (isMirrored==1 && dporduct>=0)))
	{
		correctPos[1]=ex_Pos[5].xyz;
		correctPos[2]=ex_Pos[4].xyz;
		correctPos[3]=ex_Pos[3].xyz;
		correctPos[4]=ex_Pos[2].xyz;
		correctPos[5]=ex_Pos[1].xyz;
		
		correctNrom2=-ex_Normal[4].xyz;
		correctNrom4=-ex_Normal[2].xyz;
		
		edge02=normalize(correctPos[2]-correctPos[0]);
		edge04=normalize(correctPos[4]-correctPos[0]);
		normalPrim= cross(edge02,edge04);
		lightDir=lightPos-correctPos[0];
	}
	
	vec3 edge01=normalize(correctPos[1]-correctPos[0]);
	vec3 edge05=normalize(correctPos[5]-correctPos[0]);
	vec3 edge23=normalize(correctPos[3]-correctPos[2]);
	vec3 edge24=normalize(correctPos[4]-correctPos[2]);
	dporduct=dot(lightDir,normalPrim);
	if( (isMirrored==0 && dporduct>0) || (isMirrored==1 && dporduct<0)) // primary triangle in light
	{
		
		vec3 normal012= cross(edge01,edge02);
		vec3 normal045= cross(edge04,edge05);
		vec3 normal234= cross(edge23,edge24);
		vec3 fromLight0 = normalize(correctPos[0]-lightPos);
		vec3 fromLight2 = normalize(correctPos[2]-lightPos);
		vec3 fromLight4 = normalize(correctPos[4]-lightPos);
		dporduct=dot(lightDir,normal012);
		if( (isMirrored==0 && dporduct<=0) || (isMirrored==1 && dporduct>=0)) //012 triangle is in shadow
		{
			gl_Position= VPm* vec4(correctPos[0]+correctNrom0*epsilon,1.0);
			EmitVertex();	
			gl_Position= VPm* vec4(fromLight0,0.0);
			EmitVertex();
			gl_Position= VPm* vec4(correctPos[2]+correctNrom2*epsilon,1.0);
			EmitVertex();
			gl_Position= VPm* vec4(fromLight2,0.0);
			EmitVertex();
			EndPrimitive();
		}
		dporduct=dot(lightDir,normal045);
		if( (isMirrored==0 && dporduct<=0) || (isMirrored==1 && dporduct>=0)) //045 triangle is in shadow
		{
			gl_Position= VPm* vec4(correctPos[4]+correctNrom4*epsilon,1.0);
			EmitVertex();	
			gl_Position= VPm* vec4(fromLight4,0.0);
			EmitVertex();
			gl_Position= VPm* vec4(correctPos[0]+correctNrom0*epsilon,1.0);
			EmitVertex();
			gl_Position= VPm* vec4(fromLight0,0.0);
			EmitVertex();
			EndPrimitive();
		}
		
		lightDir=lightPos-correctPos[2];
		dporduct=dot(lightDir,normal234);
		
		if( (isMirrored==0 && dporduct<=0) || (isMirrored==1 && dporduct>=0)) //234 triangle is in shadow
		{
			gl_Position= VPm* vec4(correctPos[2]+correctNrom2*epsilon,1.0);
			EmitVertex();	
			gl_Position= VPm* vec4(fromLight2,0.0);
			EmitVertex();
			gl_Position= VPm* vec4(correctPos[4]+correctNrom4*epsilon,1.0);
			EmitVertex();
			gl_Position= VPm* vec4(fromLight4,0.0);
			EmitVertex();
			EndPrimitive();
		}
	
		//front cap
		
		gl_Position= VPm* vec4(correctPos[0]+correctNrom0*epsilon,1.0);
		EmitVertex();
		gl_Position= VPm* vec4(correctPos[2]+correctNrom2*epsilon,1.0);
		EmitVertex();
		gl_Position= VPm* vec4(correctPos[4]+correctNrom4*epsilon,1.0);
		EmitVertex();
		EndPrimitive();
		
		//end cap
		gl_Position= VPm* vec4(fromLight0,0.0);
		EmitVertex();
		gl_Position= VPm* vec4(fromLight4,0.0);
		EmitVertex();
		gl_Position= VPm* vec4(fromLight2,0.0);
		EmitVertex();
		EndPrimitive();
		
		
		
	}
	
	
	
}