#version 330

in vec2 ex_TextCord;
in vec3 ex_LightDirection_tangentspace ;
in vec3 ex_CamDirection_tangentspace  ;

out vec4 out_Color;

uniform mat4 TextureCorrection;
uniform mat4 ModelMatrix;
uniform mat4 ViewMatrix;
uniform mat4 ProjectionMatrix;
uniform float time;

uniform vec4 lightColor;


uniform sampler2D reflectTexture;
uniform sampler2D refractTexture;
uniform sampler2D bumpTexture;

void main(void)
{
	float waterTransparency= 0.98;
	float reflectStrength= 0.02;
	float refrectStrength= 0.02;
	float ratioReflectRefrect= 0.4;
	
	vec2 translatedTexCord=vec2((ex_TextCord.x+time),(ex_TextCord.y+time));
	vec3 normal=normalize((texture2D(bumpTexture, translatedTexCord).grb) * 2.0 - 1.0);
	
	vec3 l = normalize(ex_LightDirection_tangentspace);
	vec3 c = normalize(ex_CamDirection_tangentspace);	
	vec3 r = reflect(-l,normal);
	float cosAlpha = clamp(dot(c,r),0,1);
	if(lightColor.a==0)
	{
		
		mat4 textureMatrix= TextureCorrection*ProjectionMatrix*ViewMatrix*ModelMatrix;
		
		vec4 textureCord4D=vec4(ex_TextCord.x,0.0,ex_TextCord.y,1.0);
		vec4 transformdOriginalTexPos=textureMatrix*textureCord4D;
		
		float waterDepth= texture2DProj(refractTexture, transformdOriginalTexPos).a;	
		reflectStrength*=waterDepth*4;
		refrectStrength*=waterDepth*4;
		
		vec4 reflTexPos=textureCord4D;
		reflTexPos.xyz+=normal*reflectStrength;
		
		vec4 refrTexPos=textureCord4D;
		refrTexPos.xyz-=normal*refrectStrength;
		
		
		vec4 reflectColor=texture2DProj(reflectTexture,textureMatrix*reflTexPos);
		vec4 refrectColor=texture2DProj(refractTexture,textureMatrix*refrTexPos);
		float waterAccAbs= pow(waterTransparency,waterDepth*100);
		refrectColor=refrectColor*(waterAccAbs)+vec4(0.0,0.5,0.5,1)*(1-waterAccAbs);
		
		float cosGama = 1-ratioReflectRefrect+clamp(dot(c,normal)*ratioReflectRefrect,0,ratioReflectRefrect);
	
		out_Color=reflectColor*(1-cosGama)+refrectColor*(cosGama);
	}
	else
	{
		out_Color=pow(cosAlpha,25)*lightColor*vec4(0.4,0.9,0.9,1);
	}
}